/*
 * A suite of programs for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */

#define FALSE	0
#define TRUE	(!FALSE)

#ifdef DEBUG
#  undef DEBUG
#  define DEBUG TRUE
#else
#  define DEBUG  FALSE
#  define NDEBUG			/* disable assert */
#endif

#include <assert.h>
#include <endian.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <string.h>

#if (__BYTE_ORDER != __LITTLE_ENDIAN && __BYTE_ORDER != __BIG_ENDIAN)
  unsupported_byte_order
#endif

#define	FATAL_ERR	1		/* generic fatal error return code */

#define forever for (;;)		/* infinite loop */
#define	min(a,b)	((a) < (b) ? (a) : (b))

typedef u_int64_t bigcnt;
typedef u_int32_t uint32;
typedef u_int8_t  byte;
typedef int       bool;

#define	OFFSET	128			/* value for binary-offset format */

/* Generic utilities */
# define xnew(n,Type)	  ((Type *)xmalloc((n)*sizeof(Type)))
# define xrnew(op,n,Type) ((op)=(Type *)xrealloc((char*)(op),(n)*sizeof(Type)))
void *xmalloc (size_t size);
void *xrealloc (void *ptr, size_t size);
void error (const char *fmt, const char *arg);
void pfatal (const char *msg);
void fatal (const char *fmt, const char *arg);

#ifndef IOR				/* buffer size ratio of stdin/stdout */
# define IOR	(1<<0)			/* a power of 2 */
#endif
char *init_stdio_bufs(void);


/* Math utilities */
#define MTSIZE	32			/* size of uint32, used by randomMT */
#define MTMAXRAND	0xFFFFFFFFu	/* max random number generated */
extern void seedMT(uint32 seed);
extern uint32 randomMT(void);
extern double gasdev(void);
extern int coin(void);

extern double invQ(double conflev);
extern double normal(void);


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
