/*
 * A suite of programs for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */

#include "codec.h"
#include "lib/harddec_opt.h"

const char *progname = PARSEOPTS_PACKAGE;


/** -= gengetopt optstart =- *******************
   --func-name=parseopts --unamed-opts
*** -= gengetopt datastart =- ******************
   package "harddec"

   version "1.5"

   purpose "Reads a sequence of binary offset byte values from standard input.
   0 is illegal, [1-127] is a -1, 128 is an erasure, [129-255] is a 1.
   Decode the stream using a hard decoder and write it on standard output
   as a bit stream in network byte order."

   option "debug" d "print some more error messages" flag off
*** -= gengetopt end =- ***********************/


int
main (int argc, char *argv[])
{
  bool debug = FALSE;

  struct gengetopt_args_info args_info;
  (void) parseopts(argc, argv, &args_info);

  if (args_info.inputs_num != 0)
    {
      parseopts_print_help();
      fatal("no arguments required", NULL);
    }
  if (args_info.debug_given)
    debug = TRUE;

  (void) init_stdio_bufs();


  /* Begin reception */
  int retval;
  byte out = 0;
  int bit = 8;
  while ((retval = getchar()) != EOF)
    {
      if (retval == 0x00)
	fatal("input stream is not in binary-offset mode", NULL);

      if (retval > OFFSET)
	out |= 1;
      else if (retval == OFFSET)
	out |= coin();
      if (--bit)
	out <<= 1;
      else
	{
	  /* Output demodulated data */
	  if (putchar(out) == EOF)
	    pfatal("error writing on output");
	  out = 0;
	  bit = 8;
	}
    }
  if (bit != 8)
    if (putchar(out >> 1) == EOF)
      pfatal("error writing on output");
  if (debug && feof(stdin))
    error("end of file reading from input", NULL);
  if (ferror(stdin))
    pfatal("error reading from input");

  return 0;
}


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
