/*
 * A suite of programs for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */

#include "../codec.h"
#include "conv.h"

/* Known codes */
struct ccode known_ccodes [] =
{
  /* Rate 1/2 codes */
  { 3, 1, 2, {0x5,0x7}, "3,1/2", "(2,1,2):5,7; Larsen 1973" },
  { 4, 1, 2, {0xb,0xf}, "4,1/2", "(2,1,3):15,17; Larsen 1973" },
  { 5, 1, 2, {0x19,0x17}, "5,1/2", "(2,1,4):23,35; Larsen 1973" },
/* Given the current data structure, the only way to associate
   puncturing patterns with convlutional codes is by disallowing
   more than one convolutional code per K,k,n triple. */
  //  { 5, 1, 2, {0x13,0x1b}, "5,1/2", "(2,1,4):31,33; maybe used in GSM" },
  { 6, 1, 2, {0x35,0x2f}, "6,1/2", "(2,1,5):53,75; Larsen 1973" },
  { 7, 1, 2, {0x6d,0x4f}, "7,1/2", "(2,1,6):133,171; NASA 1/2 standard code" },
  { 8, 1, 2, {0xe5,0x9f}, "8,1/2", "(2,1,7):247,371; Larsen 1973" },
  { 9, 1, 2, {0x11d,0x1af}, "9,1/2", "(2,1,8):753,561; IS-95 CDMA" },
  { 15, 1, 2, {0x45dd,0x69e3}, "15,1/2", "(2,1,14):56721,61713; Larsen 1973" },

  /* Rate 1/3 codes */
  { 3, 1, 3, {0x5,0x7,0x7}, "3,1/3", "(3,1,2):5,7,7; Larsen 1973" },
  { 4, 1, 3, {0xd,0xb,0xf}, "4,1/3", "(3,1,3):13,15,17; Larsen 1973" },
  { 5, 1, 3, {0x15,0x1b,0x1f}, "5,1/3", "(3,1,4):25,33,37; Larsen 1973" },
  { 6, 1, 3, {0x39,0x35,0x2f}, "6,1/3", "(3,1,5):47,53,75; Larsen 1973" },
  { 7, 1, 3, {0x4f,0x57,0x6d}, "7,1/3", "(3,1,6):171,165,133; NASA 1/3 standard code" },
  { 8, 1, 3, {0xa9,0x9b,0xef}, "8,1/3", "(3,1,7):225,331,367; Larsen 1973" },
  { 9, 1, 3, {0x1ed,0x19b,0x127}, "9,1/3", "(3,1,8):557,663,711; IS-95 CDMA" },

  /* Rate 1/6 code */
  { 15, 1, 6, {042631,047245,073363,056507,077267,064537}, "15,1/6", "(6,1,14); Mars Pathfinder" },

  { 0 }					/* terminator */
};


void
list_ccodes(FILE *fp)
{
  for (struct ccode *cp = known_ccodes; cp->K > 0; cp++)
    fprintf(fp, "%8s  %s\n", cp->name, cp->description);
}


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
