/*
 * A suite of programs for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */


/* Known codes and erasure patterns */
# define MAXn	100			/* max codeword length */

struct ccode
{
  int K;				/* constraint length */
  int k;				/* code dimension */
  int n;				/* codeword length */
  unsigned int taps[MAXn];		/* the n polynomials */
  char *name;			      /* standard name "K,k/n", e.g. "7,1/2" */
  char *description;			/* verbose description (half a line) */
};

struct ppat
{
  int K;				/* constraint length */
  int n;				/* mother code codeword length */
  int P;				/* puncturing period (symbols) */
  int q;				/* number of erasures */
  unsigned int erasures[MAXn];		/* puncturing array */
  char *name;		       /* standard name "1/n,(n*P)-q" e.g. "1/2,6-2" */
  char *description;			/* verbose description (half a line) */
};

extern struct ccode known_ccodes [];
extern struct ppat known_ppats [];

extern void list_ccodes (FILE *fp);
extern void list_ppats (FILE *fp);
extern bool *make_erasure_list (struct ppat *pp);


/* Functions for the (7,1/2) Viterbi decoder.
   Copied from viterbi27.h by Phil Karn, which see. */

#define	BFA_712(i,sym) { \
	long m0,m1;\
	/* ACS for 0 branch */\
	m0 = cmetric[i] + mets[sym];	/* 2*i */\
	m1 = cmetric[i+32] + mets[3^sym];	/* 2*i + 64 */\
	nmetric[2*i] = m0;\
	if(m1 > m0){\
		nmetric[2*i] = m1;\
		dec |= 1 << ((2*i) & 31);\
	}\
	/* ACS for 1 branch */\
	m0 -= (mets[sym] - mets[3^sym]);\
	m1 += (mets[sym] - mets[3^sym]);\
	nmetric[2*i+1] = m0;\
	if(m1 > m0){\
		nmetric[2*i+1] = m1;\
		dec |= 1 << ((2*i+1) & 31);\
	}\
}

#define	BFB_712(i,sym) { \
	long m0,m1;\
	/* ACS for 0 branch */\
	m0 = nmetric[i] + mets[sym];	/* 2*i */\
	m1 = nmetric[i+32] + mets[3^sym]; /* 2*i + 64 */\
	cmetric[2*i] = m0;\
	if(m1 > m0){\
		cmetric[2*i] = m1;\
		dec |= 1 << ((2*i) & 31);\
	}\
	/* ACS for 1 branch */\
	m0 -= (mets[sym] - mets[3^sym]);\
	m1 += (mets[sym] - mets[3^sym]);\
	cmetric[2*i+1] = m0;\
	if(m1 > m0){\
		cmetric[2*i+1] = m1;\
		dec |= 1 << ((2*i+1) & 31);\
	}\
}


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
