/*
 * A suite of programs for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */

#include "../codec.h"
#include "conv.h"

/* Known erasure patterns */
struct ppat known_ppats [] =
{
  /* Patterns used with 1/2 mother codes */
  { 7, 2, 2, 1, {0x3,0x2},   "7,1/2=2/3", "4-1; IEEE 802.11 for rate 2/3" },
  { 7, 2, 3, 2, {0x6,0x5},   "7,1/2=3/4", "6-2; IEEE 802.11 for rate 3/4" },
  { 7, 2, 4, 3, {0xf,0x8},   "7,1/2=4/5", "8-3; IEEE 802.11 for rate 4/5" },
  { 7, 2, 5, 4, {0x1a,0x15}, "7,1/2=5/6", "10-4; IEEE 802.11 for rate 5/6" },
  { 7, 2, 6, 5, {0x3a,0x25}, "7,1/2=6/7", "12-5; IEEE 802.11 for rate 6/7" },
  { 7, 2, 7, 6, {0x7a,0x45}, "7,1/2=7/8", "14-6; IEEE 802.11 for rate 7/8" },

  { 0 }					/* terminator */
};


void
list_ppats(FILE *fp)
{
  for (struct ppat *pp = known_ppats; pp->K > 0; pp++)
    fprintf(fp, "%12s  %s\n", pp->name, pp->description);
}

/* Take an erasure pattern structure and build the corresponding
   erasure list, where 1 means no erasure, and 0 means erasure. */
bool *
make_erasure_list(struct ppat *pp)
{
  bool *list = xnew(pp->n * pp->P, bool);
  int i = 0;
  for (int s = pp->P-1; s >= 0; s--)	/* period (symbols) */
    for (int b = 0; b < pp->n; b++)	/* bits per symbol */
      list[i++] = (pp->erasures[b] >> s) & 1;
  return list;
}


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
