/*
 * A program for simulating channel codings
 * Copyright (C) 2002 Francesco Potort
 * See file COPYING for details
 */

#include "../codec.h"

/* Allocate the stdin and stdout buffers, on a page boundary.
   The respective sizes are pgsz and pgsz/IOR. */
char *
init_stdio_bufs(void)
{
  size_t pgsz = getpagesize();
  if (DEBUG)
    {
      size_t check = pgsz;
      while (((check >>= 1) & 1) == 0) ;
      assert(pgsz > 511 && check == 1);	/* sanity check: pgsz a power of 2 */
    }
  char *buf = xnew(2*pgsz-1+pgsz/IOR, char);
  size_t misalign = (size_t)buf & (pgsz-1);
  if (misalign != 0)			/* if not aligned on page boundary */
    buf += pgsz-misalign;		/* realign it */
  setbuffer(stdin, buf, pgsz);		/* set buffered input buffer */
  setbuffer(stdout, buf+pgsz, pgsz/IOR); /* set buffered output buffer */

  return buf;
}

/* Print error message.  `s1' is printf control string, `s2' is arg for it. */
void
error (const char *fmt,
       const char *arg)
{
  extern const char *progname;
  fprintf (stderr, "%s: ", progname);
  fprintf (stderr, fmt, arg);
  fprintf (stderr, "\n");
}

void
pfatal (const char *msg)
{
  perror (msg);
  exit (FATAL_ERR);
}

/* Print error message and exit.  */
void
fatal (const char *fmt,
       const char *arg)
{
  error (fmt, arg);
  exit (FATAL_ERR);
}

/* Like malloc but get fatal error if memory is exhausted.  */
void *
xmalloc (size_t size)
{
  void *result = malloc (size);
  if (result == NULL)
    fatal ("virtual memory exhausted", (char *)NULL);
  return result;
}

void *
xrealloc (void *ptr,
	  size_t size)
{
  void *result = realloc (ptr, size);
  if (result == NULL)
    fatal ("virtual memory exhausted", (char *)NULL);
  return result;
}


/*
 * Local Variables:
 * comment-column: 40
 * fill-column: 79
 * c-font-lock-extra-types: ("FILE" "\\sw+_t" "bigcnt" "uint32" "byte" "bool")
 * End:
 */
