#! /bin/bash
#  A suite of programs for simulating channel codings
#  Copyright (C) 2002,2003,2005 Francesco Potort
#  See file COPYING for details

if [ $# -lt 3 ]; then
    cat <<EOF
usage: $0 <range> <outfile> <errfile>
	range is <min>:<step>:<max> in dB
	outfile and errfile can be - for stdin and stdout
	see the source for the available options
EOF
    exit 2
fi
LC_NUMERIC=C				# use C style for numbers

errlimit=10				# default errors to generate
convcode='(7,1/2)'			# default convolutional code
conflev=0.9				# default confidence level
qlevopt=				# default is hard decoding

while getopts "e:l:q:" opt; do
    case $opt in
    "e")
	    errlimit="$OPTARG" ;;
    "l")
	    conflev="$OPTARG" ;;
    *)
	    exit 1 ;;
    esac
done
shift $((OPTIND-1))			# shift options away
range=$1				# dB range as start:step:limit
out=$2					# output file
errors=$3				# errors file

start=$(echo $range | cut -d: -f1)	# min dB
step=$(echo $range | cut -d: -f2)	# dB step
limit=$(echo $range | cut -d: -f3)	# max dB


gain=0					# code gain in dB
    
if [ -n "$out" -a "$out" != - ]; then
    : rewind out file > $out
else
    out=/dev/stdout
fi
exec 3>$out; out=/dev/fd/3
if [ "$errors" ]; then
    : rewind errors file > $errors
fi

epoch=$(date +%s)			# get starting time
bitcount=.TMPcodec.bitcount.$$		# contains total number of bits
trap "rm -f $bitcount" EXIT

printf "BER test for the uncoded channel with %.0f%% confidence: \n" \
       $(echo "$conflev * 100"|bc -l) >>$out
printf " Eb/No Theoretical            Measured \n" >>$out

bits=0;
ebno=$start;
while [ $(echo "$ebno <= $limit"|bc -l) = 1 ]; do

    ecno=$(echo "$ebno-$gain"|bc -l)	# channel Eb/No
    theor=$(echo "std=sqrt(e($ecno/10*l(10))); t=1/(1+0.5*std);" \
		 "erfc=t*e( -std*std + ((((((((0.17087277 *t-0.82215223)*t" \
		 "+1.48851587)*t-1.13520398)*t+0.27886807)*t-0.18628806)*t" \
		 "+0.09678418)*t+0.37409196)*t+1.00002368)*t-1.26551223);" \
		 "0.5*erfc"|bc -l)

    printf "%6.2f %9.2e" $ebno $theor >>$out

    rangen -b |
	channel --noise=awgn,$ecno |
	harddec |
	printerr -l $conflev -e $errlimit <(rangen) - $out - |
	tee -a $errors |
	tail -1 | cut -f 2 > $bitcount

    bits=$(echo "$bits+$(cat $bitcount)"|bc -l)

    ebno=$(echo "$ebno + $step"|bc -l)
done

# Print elapsed time and estimated speed
{
  rt=$(($(date +%s)-epoch))
  echo -n Total $bits generated symbols, runtime
  printf " %02d:%02d hours, sym/hour %.2e.\n" \
	 $((rt/3600)) $(((rt/60)%60)) $(echo "$bits/$rt*3600"|bc -l)
} >>$out
